
Imports System.Collections.Generic
Imports System.Text

Namespace Generic
    Partial Public Class Deque(Of T)

        <Serializable()> _
        Private Class Enumerator
            Implements IEnumerator(Of T)
            Private owner As Deque(Of T)

            Private currentNode As Node

            Private m_current As T = Nothing

            Private moveResult As Boolean = False

            Private version As Long

            ' A value indicating whether the enumerator has been disposed.
            Private disposed As Boolean = False

            Public Sub New(ByVal owner As Deque(Of T))
                Me.owner = owner
                currentNode = owner.front
                Me.version = owner.version
            End Sub

            Public Sub Reset() Implements System.Collections.IEnumerator.Reset
                If disposed Then
                    Throw New ObjectDisposedException(Me.[GetType]().Name)
                ElseIf version <> owner.version Then
                    Throw New InvalidOperationException("The Deque was modified after the enumerator was created.")
                End If

                currentNode = owner.front
                moveResult = False
            End Sub

            ' TODO ------------------------------------------------------------------------------- Livio - RIMOSSO
            Public ReadOnly Property Current() As Object Implements IEnumerator(Of T).Current ', System.Collections.IEnumerator.Current
                Get
                    If disposed Then
                        Throw New ObjectDisposedException(Me.[GetType]().Name)
                    ElseIf Not moveResult Then
                        Throw New InvalidOperationException("The enumerator is positioned before the first " & "element of the Deque or after the last element.")
                    End If

                    Return m_current
                End Get
            End Property

            Public Function MoveNext() As Boolean Implements System.Collections.IEnumerator.MoveNext
                If disposed Then
                    Throw New ObjectDisposedException(Me.[GetType]().Name)
                ElseIf version <> owner.version Then
                    Throw New InvalidOperationException("The Deque was modified after the enumerator was created.")
                End If

                If currentNode IsNot Nothing Then
                    m_current = currentNode.Value
                    currentNode = currentNode.[Next]

                    moveResult = True
                Else
                    moveResult = False
                End If

                Return moveResult
            End Function

            Private ReadOnly Property IEnumerator_Current() As T Implements IEnumerator(Of T).Current
                Get
                    If disposed Then
                        Throw New ObjectDisposedException(Me.[GetType]().Name)
                    ElseIf Not moveResult Then
                        Throw New InvalidOperationException("The enumerator is positioned before the first " & "element of the Deque or after the last element.")
                    End If

                    Return m_current
                End Get
            End Property

            Public Sub Dispose() Implements IDisposable.Dispose
                disposed = True
            End Sub

        End Class

    End Class

End Namespace
