


Public Class TimeSignatureBuilder
    Implements IMessageBuilder

    Private Const DefaultNumerator As Byte = 4
    Private Const DefaultDenominator As Byte = 2
    Private Const DefaultClocksPerMetronomeClick As Byte = 24
    Private Const DefaultThirtySecondNotesPerQuarterNote As Byte = 8

    Private data As Byte() = New Byte(MetaMessage.TimeSigLength - 1) {}
    Private m_result As MetaMessage = Nothing
    Private changed As Boolean = True

    Public Sub New()
        Numerator = DefaultNumerator
        Denominator = DefaultDenominator
        ClocksPerMetronomeClick = DefaultClocksPerMetronomeClick
        ThirtySecondNotesPerQuarterNote = DefaultThirtySecondNotesPerQuarterNote
    End Sub

    Public Sub New(ByVal message As MetaMessage)
        Initialize(message)
    End Sub

    Public Sub Initialize(ByVal message As MetaMessage)
        If message.MetaType <> MetaType.TimeSignature Then
            Throw New ArgumentException("Wrong meta event type.", "message")
        End If
        data = message.GetBytes()
    End Sub


    Public Property Numerator() As Byte
        Get
            Return data(0)
        End Get
        Set(ByVal value As Byte)
            If value < 1 Then
                Throw New ArgumentOutOfRangeException("Numerator", value, "Numerator out of range.")
            End If
            data(0) = value
            changed = True
        End Set
    End Property

    Public Property Denominator() As Byte
        Get
            Return Convert.ToByte(Math.Pow(2, data(1)))
        End Get
        Set(ByVal value As Byte)
            If value < 2 OrElse value > 32 Then
                Throw New ArgumentOutOfRangeException("Denominator must be between 2 and 32.")
            ElseIf (value And (value - 1)) <> 0 Then
                Throw New ArgumentException("Denominator must be a power of 2.")
            End If
            data(1) = Convert.ToByte(Math.Log(value, 2))
            changed = True
        End Set
    End Property

    Public Property ClocksPerMetronomeClick() As Byte
        Get
            Return data(2)
        End Get
        Set(ByVal value As Byte)
            data(2) = value
            changed = True
        End Set
    End Property

    Public Property ThirtySecondNotesPerQuarterNote() As Byte
        Get
            Return data(3)
        End Get
        Set(ByVal value As Byte)
            data(3) = value
            changed = True
        End Set
    End Property

    Public ReadOnly Property Result() As MetaMessage
        Get
            Return m_result
        End Get
    End Property

    Public Sub Build() Implements IMessageBuilder.Build
        If changed Then
            m_result = New MetaMessage(MetaType.TimeSignature, data)
            changed = False
        End If
    End Sub

End Class
