

Imports System.Collections

Public Class ChannelChaser

    Private controllerMessages As ChannelMessage(,)
    Private programChangeMessages As ChannelMessage()
    Private pitchBendMessages As ChannelMessage()
    Private channelPressureMessages As ChannelMessage()
    Private polyPressureMessages As ChannelMessage()
    Public Event Chased As EventHandler(Of ChasedEventArgs)

    Public Sub New()
        Dim c As Integer = ChannelMessage.MidiChannelMaxValue + 1
        Dim d As Integer = ShortMessage.DataMaxValue + 1

        controllerMessages = New ChannelMessage(c - 1, d - 1) {}
        programChangeMessages = New ChannelMessage(c - 1) {}
        pitchBendMessages = New ChannelMessage(c - 1) {}
        channelPressureMessages = New ChannelMessage(c - 1) {}
        polyPressureMessages = New ChannelMessage(c - 1) {}
    End Sub

    Public Sub Process(ByVal message As ChannelMessage)
        Select Case message.Command
            Case ChannelCommand.Controller
                controllerMessages(message.MidiChannel, message.Data1) = message

            Case ChannelCommand.ChannelPressure
                channelPressureMessages(message.MidiChannel) = message

            Case ChannelCommand.PitchWheel
                pitchBendMessages(message.MidiChannel) = message

            Case ChannelCommand.PolyPressure
                polyPressureMessages(message.MidiChannel) = message

            Case ChannelCommand.ProgramChange
                programChangeMessages(message.MidiChannel) = message

        End Select
    End Sub

    Public Sub Chase()
        Dim chasedMessages As New ArrayList()
        '
        For c As Integer = 0 To ChannelMessage.MidiChannelMaxValue
            For n As Integer = 0 To ShortMessage.DataMaxValue
                If controllerMessages(c, n) IsNot Nothing Then
                    chasedMessages.Add(controllerMessages(c, n))
                    controllerMessages(c, n) = Nothing
                End If
            Next
            If programChangeMessages(c) IsNot Nothing Then
                chasedMessages.Add(programChangeMessages(c))
                programChangeMessages(c) = Nothing
            End If
            If pitchBendMessages(c) IsNot Nothing Then
                chasedMessages.Add(pitchBendMessages(c))
                pitchBendMessages(c) = Nothing
            End If
            If channelPressureMessages(c) IsNot Nothing Then
                chasedMessages.Add(channelPressureMessages(c))
                channelPressureMessages(c) = Nothing
            End If
            If polyPressureMessages(c) IsNot Nothing Then
                chasedMessages.Add(polyPressureMessages(c))
                polyPressureMessages(c) = Nothing
            End If
        Next
        '
        OnChased(New ChasedEventArgs(chasedMessages))
    End Sub

    Public Sub Reset()
        For c As Integer = 0 To ChannelMessage.MidiChannelMaxValue
            For n As Integer = 0 To ShortMessage.DataMaxValue
                controllerMessages(c, n) = Nothing
            Next
            programChangeMessages(c) = Nothing
            pitchBendMessages(c) = Nothing
            channelPressureMessages(c) = Nothing
            polyPressureMessages(c) = Nothing
        Next
    End Sub

    Protected Overridable Sub OnChased(ByVal e As ChasedEventArgs)
        RaiseEvent Chased(Me, e)
    End Sub

End Class
