

Imports System.Diagnostics

Partial Public NotInheritable Class Track
    <Conditional("DEBUG")> _
    Public Shared Sub Test()
        TestInsert()
        TestRemoveAt()
        TestMerge()
    End Sub

    <Conditional("DEBUG")> _
    Private Shared Sub TestInsert()
        Dim track As New Track()
        Dim midiEventCount As Integer = 2000
        Dim positionMax As Integer = 32000
        Dim endOfTrackOffset As Integer = 1000
        Dim length As Integer = 0
        Dim position As Integer = 0
        Dim message As New ChannelMessage(ChannelCommand.NoteOff, 0, 60, 0)
        Dim r As New Random()

        For i As Integer = 0 To midiEventCount - 1
            position = r.[Next](positionMax)

            If position > length Then
                length = position
            End If

            track.Insert(position, message)
        Next

        track.EndOfTrackOffset = endOfTrackOffset

        length += track.EndOfTrackOffset

        Debug.Assert(track.Count = midiEventCount + 1)
        Debug.Assert(track.Length = length)
    End Sub

    <Conditional("DEBUG")> _
    Private Shared Sub TestRemoveAt()
        Dim a As New Track()
        Dim message As New ChannelMessage(ChannelCommand.NoteOff, 0, 60, 0)

        a.Insert(0, message)
        a.Insert(10, message)
        a.Insert(20, message)
        a.Insert(30, message)
        a.Insert(40, message)

        Dim count As Integer = a.Count

        a.RemoveAt(0)

        Debug.Assert(a.Count = count - 1)

        a.RemoveAt(a.Count - 2)

        Debug.Assert(a.Count = count - 2)
        Debug.Assert(a.GetMidiEvent(0).AbsoluteTicks = 10)
        Debug.Assert(a.GetMidiEvent(a.Count - 2).AbsoluteTicks = 30)

        a.RemoveAt(0)
        a.RemoveAt(0)
        a.RemoveAt(0)

        Debug.Assert(a.Count = 1)
    End Sub

    <Conditional("DEBUG")> _
    Private Shared Sub TestMerge()
        Dim a As New Track()
        Dim b As New Track()

        a.Merge(b)

        Debug.Assert(a.Count = 1)

        Dim message As New ChannelMessage(ChannelCommand.NoteOff, 0, 60, 0)

        b.Insert(0, message)
        b.Insert(10, message)
        b.Insert(20, message)
        b.Insert(30, message)
        b.Insert(40, message)

        a.Merge(b)

        Debug.Assert(a.Count = 1 + b.Count - 1)

        a.Clear()

        Debug.Assert(a.Count = 1)

        a.Insert(0, message)
        a.Insert(10, message)
        a.Insert(20, message)
        a.Insert(30, message)
        a.Insert(40, message)

        Dim count As Integer = a.Count

        a.Merge(b)

        Debug.Assert(a.Count = count + b.Count - 1)
        Debug.Assert(a.GetMidiEvent(0).DeltaTicks = 0)
        Debug.Assert(a.GetMidiEvent(1).DeltaTicks = 0)
        Debug.Assert(a.GetMidiEvent(2).DeltaTicks = 10)
        Debug.Assert(a.GetMidiEvent(3).DeltaTicks = 0)
        Debug.Assert(a.GetMidiEvent(4).DeltaTicks = 10)
        Debug.Assert(a.GetMidiEvent(5).DeltaTicks = 0)
        Debug.Assert(a.GetMidiEvent(6).DeltaTicks = 10)
        Debug.Assert(a.GetMidiEvent(7).DeltaTicks = 0)
        Debug.Assert(a.GetMidiEvent(8).DeltaTicks = 10)
        Debug.Assert(a.GetMidiEvent(9).DeltaTicks = 0)
    End Sub

End Class
