

Imports System.Collections.Generic
Imports System.Drawing
Imports System.Windows.Forms
Imports LC.Multimedia

Namespace UI

    Partial Public Class PianoControl
        Private Class PianoKey
            Inherits Control

            Private owner As PianoControl
            Private m_BlackKey As Boolean = False
            Private [on] As Boolean = False
            Private onBrush As New SolidBrush(Color.SkyBlue)
            Private onBlackBrush As New SolidBrush(Color.DarkBlue)
            Private offBrush As New SolidBrush(Color.White)
            Private m_noteID As Integer = 60

            Public Sub New(ByVal owner As PianoControl)
                Me.owner = owner
                Me.TabStop = False
            End Sub

            Public Sub PressPianoKey()
                If [on] Then
                    Return
                End If
                [on] = True

                Invalidate()

                owner.OnPianoKeyDown(New PianoKeyEventArgs(m_noteID))
            End Sub

            Public Sub ReleasePianoKey()
                If Not [on] Then
                    Return
                End If

                [on] = False

                Invalidate()

                owner.OnPianoKeyUp(New PianoKeyEventArgs(m_noteID))
            End Sub

            Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
                If disposing Then
                    onBrush.Dispose()
                    offBrush.Dispose()
                End If
                MyBase.Dispose(disposing)
            End Sub

            Protected Overloads Overrides Sub OnMouseEnter(ByVal e As EventArgs)
                If Control.MouseButtons = MouseButtons.Left Then
                    PressPianoKey()
                End If
                MyBase.OnMouseEnter(e)
            End Sub

            Protected Overloads Overrides Sub OnMouseLeave(ByVal e As EventArgs)
                If [on] Then
                    ReleasePianoKey()
                End If
                MyBase.OnMouseLeave(e)
            End Sub

            Protected Overloads Overrides Sub OnMouseDown(ByVal e As MouseEventArgs)
                PressPianoKey()
                If Not owner.Focused Then
                    owner.Focus()
                End If
                MyBase.OnMouseDown(e)
            End Sub

            Protected Overloads Overrides Sub OnMouseUp(ByVal e As MouseEventArgs)
                ReleasePianoKey()
                MyBase.OnMouseUp(e)
            End Sub

            Protected Overloads Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)
                If e.X < 0 OrElse e.X > Width OrElse e.Y < 0 OrElse e.Y > Height Then
                    Capture = False
                End If
                MyBase.OnMouseMove(e)
            End Sub

            Protected Overloads Overrides Sub OnPaint(ByVal e As PaintEventArgs)
                If m_BlackKey Then
                    If [on] Then
                        e.Graphics.FillRectangle(onBlackBrush, 0, 0, Size.Width, Size.Height)
                        e.Graphics.DrawLine(Pens.Gray, 2, 4, 2, Size.Height - 7)
                    Else
                        e.Graphics.FillRectangle(offBrush, 0, 0, Size.Width, Size.Height)
                        e.Graphics.DrawLine(Pens.DarkGray, 2, 3, 2, Size.Height - 8)
                    End If
                Else
                    If [on] Then
                        e.Graphics.FillRectangle(onBrush, 0, 0, Size.Width, Size.Height)
                    Else
                        e.Graphics.FillRectangle(offBrush, 0, 0, Size.Width, Size.Height)
                    End If
                    e.Graphics.DrawRectangle(Pens.Black, 0, 0, Size.Width - 1, Size.Height - 1)
                End If
         
                MyBase.OnPaint(e)
            End Sub

            Public Property NoteOnColor() As Color
                Get
                    Return onBrush.Color
                End Get
                Set(ByVal value As Color)
                    onBrush.Color = value
                    If [on] Then
                        Invalidate()
                    End If
                End Set
            End Property

            Public Property NoteBlackOnColor() As Color
                Get
                    Return onBlackBrush.Color
                End Get
                Set(ByVal value As Color)
                    onBlackBrush.Color = value
                    If [on] Then
                        Invalidate()
                    End If
                End Set
            End Property

            Public Property NoteOffColor() As Color
                Get
                    Return offBrush.Color
                End Get
                Set(ByVal value As Color)
                    offBrush.Color = value
                    If Not [on] Then
                        Invalidate()
                    End If
                End Set
            End Property

            Public Property NoteID() As Integer
                Get
                    Return m_noteID
                End Get
                Set(ByVal value As Integer)
                    If value < 0 OrElse value > ShortMessage.DataMaxValue Then
                        Throw New ArgumentOutOfRangeException("NoteID", m_noteID, "Note ID out of range.")
                    End If
                    m_noteID = value
                End Set
            End Property

            Public ReadOnly Property IsPianoKeyPressed() As Boolean
                Get
                    Return [on]
                End Get
            End Property

            Public Property IsBlackKey() As Boolean
                Get
                    Return m_BlackKey
                End Get
                Set(ByVal value As Boolean)
                    m_BlackKey = value
                End Set
            End Property

        End Class

    End Class

End Namespace
