

Imports System.Threading


Public Class AsyncResult
	Implements IAsyncResult

    ' The owner of this AsyncResult object.
    Private m_owner As Object

    ' The callback to be invoked when the operation completes.
    Private callback As AsyncCallback

    ' User state information.
    Private state As Object

    ' For signaling when the operation has completed.
    Private waitHandle As New ManualResetEvent(False)

    ' A value indicating whether the operation completed synchronously.
    Private m_completedSynchronously As Boolean

    ' A value indicating whether the operation has completed.
    Private m_isCompleted As Boolean = False

    ' The ID of the thread this AsyncResult object originated on.
    Private threadId As Integer



    Public Sub New(ByVal owner As Object, ByVal callback As AsyncCallback, ByVal state As Object)
        Me.m_owner = owner
        Me.callback = callback
        Me.state = state

        ' Get the current thread ID. This will be used later to determine
        ' if the operation completed synchronously.
        threadId = Thread.CurrentThread.ManagedThreadId
    End Sub


    ' --------------------------------------------------------- signals that the operation has completed
    Public Sub Signal()
        m_isCompleted = True
        m_completedSynchronously = (threadId = Thread.CurrentThread.ManagedThreadId)
        waitHandle.[Set]()
        If callback IsNot Nothing Then
            callback(Me)
        End If
    End Sub


    Public ReadOnly Property Owner() As Object
        Get
            Return m_owner
        End Get
    End Property


    Public ReadOnly Property AsyncState() As Object Implements IAsyncResult.AsyncState
        Get
            Return state
        End Get
    End Property

    Public ReadOnly Property AsyncWaitHandle() As WaitHandle Implements IAsyncResult.AsyncWaitHandle
        Get
            Return waitHandle
        End Get
    End Property

    Public ReadOnly Property CompletedSynchronously() As Boolean Implements IAsyncResult.CompletedSynchronously
        Get
            Return m_completedSynchronously
        End Get
    End Property

    Public ReadOnly Property IsCompleted() As Boolean Implements IAsyncResult.IsCompleted
        Get
            Return m_isCompleted
        End Get
    End Property

End Class
