
Imports System.Diagnostics
Imports System.Threading

Partial Public Class DelegateQueue

    Private Enum NotificationType
        None
        BeginInvokeCompleted
        PostCompleted
    End Enum


    ' ---------------------------------------------------------- IAsyncResult interface for the DelegateQueue class
    Private Class DelegateQueueAsyncResult
        Inherits AsyncResult

        ' ------------------------------------------------------ the delegate to be invoked
        Private m_method As [Delegate]
        ' ------------------------------------------------------ args to be passed to the delegate
        Private args As Object()
        ' ------------------------------------------------------ the object returned from the delegate
        Private m_returnValue As Object = Nothing
        ' ------------------------------------------------------ a possible exception thrown by invoking the method
        Private m_error As Exception = Nothing
        ' ------------------------------------------------------ 
        Private m_notificationType As NotificationType


        Public Sub New(ByVal owner As Object, ByVal method As [Delegate], ByVal args As Object(), ByVal synchronously As Boolean, ByVal notificationType As NotificationType)
            MyBase.New(owner, Nothing, Nothing)
            Me.m_method = method
            Me.args = args
            Me.m_notificationType = notificationType
        End Sub

        Public Sub New(ByVal owner As Object, ByVal callback As AsyncCallback, ByVal state As Object, ByVal method As [Delegate], ByVal args As Object(), ByVal synchronously As Boolean, _
         ByVal notificationType As NotificationType)
            MyBase.New(owner, callback, state)
            Me.m_method = method
            Me.args = args
            Me.m_notificationType = notificationType
        End Sub

        Public Sub Invoke()
            Try
                m_returnValue = m_method.DynamicInvoke(args)
            Catch ex As Exception
                m_error = ex
            Finally
                Signal()
            End Try
        End Sub

        Public Function GetArgs() As Object()
            Return args
        End Function

        Public ReadOnly Property ReturnValue() As Object
            Get
                Return m_returnValue
            End Get
        End Property

        Public Property [Error]() As Exception
            Get
                Return m_error
            End Get
            Set(ByVal value As Exception)
                m_error = Value
            End Set
        End Property

        Public ReadOnly Property Method() As [Delegate]
            Get
                Return m_method
            End Get
        End Property

        Public ReadOnly Property NotificationType() As NotificationType
            Get
                Return m_notificationType
            End Get
        End Property

    End Class

End Class
